@@ -33,6 +33,7 @@
 #include <dbus/dbus-glib.h>
 
 #include <telepathy-glib/account-manager.h>
+#include <telepathy-glib/defs.h>
 #include <telepathy-glib/util.h>
 
 #include <libempathy/empathy-utils.h>
@@ -71,8 +72,6 @@
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyAccountsDialog)
 G_DEFINE_TYPE (EmpathyAccountsDialog, empathy_accounts_dialog, GTK_TYPE_DIALOG);
 
-static EmpathyAccountsDialog *dialog_singleton = NULL;
-
 typedef struct {
   GtkWidget *alignment_settings;
   GtkWidget *alignment_infobar;
@@ -132,8 +131,8 @@ typedef struct {
 enum {
   COL_NAME,
   COL_STATUS,
-  COL_ACCOUNT_POINTER,
-  COL_ACCOUNT_SETTINGS_POINTER,
+  COL_ACCOUNT,
+  COL_ACCOUNT_SETTINGS,
   COL_COUNT
 };
 
@@ -174,6 +173,10 @@ static void accounts_dialog_presence_changed_cb (TpAccount *account,
     gchar *status_message,
     EmpathyAccountsDialog *dialog);
 
+static void accounts_dialog_model_selection_changed (
+    GtkTreeSelection *selection,
+    EmpathyAccountsDialog *dialog);
+
 static void
 accounts_dialog_update_name_label (EmpathyAccountsDialog *dialog,
     const gchar *display_name)
@@ -213,8 +216,9 @@ accounts_dialog_update_status_infobar (EmpathyAccountsDialog *dialog,
   if (!gtk_tree_selection_get_selected (selection, &model, &iter))
     return;
 
-  gtk_tree_model_get (model, &iter,
-      COL_ACCOUNT_POINTER, &selected_account, -1);
+  gtk_tree_model_get (model, &iter, COL_ACCOUNT, &selected_account, -1);
+  if (selected_account != NULL)
+    g_object_unref (selected_account);
 
   /* do not update the infobar when the account is not selected */
   if (account != selected_account)
@@ -362,8 +366,8 @@ empathy_account_dialog_widget_cancelled_cb (
     return;
 
   gtk_tree_model_get (model, &iter,
-      COL_ACCOUNT_SETTINGS_POINTER, &settings,
-      COL_ACCOUNT_POINTER, &account, -1);
+      COL_ACCOUNT_SETTINGS, &settings,
+      COL_ACCOUNT, &account, -1);
 
   empathy_account_widget_discard_pending_changes (priv->setting_widget_object);
 
@@ -394,19 +398,10 @@ empathy_account_dialog_account_created_cb (EmpathyAccountWidget *widget_object,
     TpAccount *account,
     EmpathyAccountsDialog *dialog)
 {
-  gchar *display_name;
   EmpathyAccountSettings *settings =
       accounts_dialog_model_get_selected_settings (dialog);
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
 
-  display_name = empathy_account_widget_get_default_display_name (
-      widget_object);
-
-  empathy_account_settings_set_display_name_async (settings,
-      display_name, NULL, NULL);
-
-  g_free (display_name);
-
   accounts_dialog_update_settings (dialog, settings);
   accounts_dialog_update_status_infobar (dialog,
       empathy_account_settings_get_account (settings));
@@ -425,6 +420,28 @@ empathy_account_dialog_account_created_cb (EmpathyAccountWidget *widget_object,
     g_object_unref (settings);
 }
 
+static gboolean
+accounts_dialog_has_valid_accounts (EmpathyAccountsDialog *dialog)
+{
+  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+  gboolean creating;
+
+  g_object_get (priv->setting_widget_object,
+      "creating-account", &creating, NULL);
+
+  if (!creating)
+    return TRUE;
+
+  model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
+
+  if (gtk_tree_model_get_iter_first (model, &iter))
+    return gtk_tree_model_iter_next (model, &iter);
+
+  return FALSE;
+}
+
 static void
 account_dialog_create_settings_widget (EmpathyAccountsDialog *dialog,
     EmpathyAccountSettings *settings)
@@ -436,6 +453,10 @@ account_dialog_create_settings_widget (EmpathyAccountsDialog *dialog,
   priv->setting_widget_object =
       empathy_account_widget_new_for_protocol (settings, FALSE);
 
+  if (accounts_dialog_has_valid_accounts (dialog))
+    empathy_account_widget_set_other_accounts_exist (
+        priv->setting_widget_object, TRUE);
+
   priv->settings_widget =
       empathy_account_widget_get_widget (priv->setting_widget_object);
 
@@ -516,7 +537,7 @@ accounts_dialog_has_pending_change (EmpathyAccountsDialog *dialog,
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
 
   if (gtk_tree_selection_get_selected (selection, &model, &iter))
-    gtk_tree_model_get (model, &iter, COL_ACCOUNT_POINTER, account, -1);
+    gtk_tree_model_get (model, &iter, COL_ACCOUNT, account, -1);
 
   return priv->setting_widget_object != NULL
       && empathy_account_widget_contains_pending_changes (
@@ -529,19 +550,25 @@ accounts_dialog_setup_ui_to_add_account (EmpathyAccountsDialog *dialog)
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
   EmpathyAccountSettings *settings;
   gchar *str;
-  const gchar *display_name;
+  const gchar *name, *display_name;
   TpConnectionManager *cm;
   TpConnectionManagerProtocol *proto;
-  gboolean is_gtalk;
+  gboolean is_gtalk, is_facebook;
 
   cm = empathy_protocol_chooser_dup_selected (
-      EMPATHY_PROTOCOL_CHOOSER (priv->combobox_protocol), &proto, &is_gtalk);
+      EMPATHY_PROTOCOL_CHOOSER (priv->combobox_protocol), &proto, &is_gtalk,
+      &is_facebook);
   if (cm == NULL)
     return;
 
-  display_name = empathy_protocol_name_to_display_name (
-      is_gtalk ? "gtalk" : proto->name);
+  if (is_gtalk)
+    name = "gtalk";
+  else if (is_facebook)
+    name ="facebook";
+  else
+    name = proto->name;
 
+  display_name = empathy_protocol_name_to_display_name (name);
   if (display_name == NULL)
     display_name = proto->name;
 
@@ -557,6 +584,9 @@ accounts_dialog_setup_ui_to_add_account (EmpathyAccountsDialog *dialog)
   if (is_gtalk)
     empathy_account_settings_set_icon_name_async (settings, "im-google-talk",
         NULL, NULL);
+  else if (is_facebook)
+    empathy_account_settings_set_icon_name_async (settings, "im-facebook",
+        NULL, NULL);
 
   accounts_dialog_add (dialog, settings);
   accounts_dialog_model_set_selected (dialog, settings);
@@ -610,8 +640,17 @@ accounts_dialog_protocol_changed_cb (GtkWidget *widget,
   /* We are creating a new widget to replace the current one, don't ask
    * confirmation to the user. */
   priv->force_change_row = TRUE;
+
+  /* We'll update the selection after we create the new account widgets;
+   * updating it right now causes problems for the # of accounts = zero case */
+  g_signal_handlers_block_by_func (selection,
+      accounts_dialog_model_selection_changed, dialog);
+
   gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
 
+  g_signal_handlers_unblock_by_func (selection,
+      accounts_dialog_model_selection_changed, dialog);
+
   accounts_dialog_setup_ui_to_add_account (dialog);
 
   /* Restore "account" and "password" parameters in the new widget */
@@ -734,6 +773,9 @@ accounts_dialog_button_add_clicked_cb (GtkWidget *button,
       gtk_widget_set_sensitive (priv->button_remove, FALSE);
       gtk_widget_set_sensitive (priv->button_import, FALSE);
     }
+
+  if (account != NULL)
+    g_object_unref (account);
 }
 
 static void
@@ -885,7 +927,7 @@ accounts_dialog_model_status_pixbuf_data_func (GtkTreeViewColumn *tree_column,
 {
   TpAccount *account;
 
-  gtk_tree_model_get (model, iter, COL_ACCOUNT_POINTER, &account, -1);
+  gtk_tree_model_get (model, iter, COL_ACCOUNT, &account, -1);
 
   g_object_set (cell,
       "icon-name", get_status_icon_for_account (dialog, account),
@@ -909,7 +951,7 @@ accounts_dialog_model_protocol_pixbuf_data_func (GtkTreeViewColumn *tree_column,
 
   gtk_tree_model_get (model, iter,
       COL_STATUS, &status,
-      COL_ACCOUNT_SETTINGS_POINTER, &settings,
+      COL_ACCOUNT_SETTINGS, &settings,
       -1);
 
   icon_name = empathy_account_settings_get_icon_name (settings);
@@ -980,7 +1022,7 @@ accounts_dialog_name_edited_cb (GtkCellRendererText *renderer,
   treepath = gtk_tree_path_new_from_string (path);
   gtk_tree_model_get_iter (model, &iter, treepath);
   gtk_tree_model_get (model, &iter,
-      COL_ACCOUNT_SETTINGS_POINTER, &settings,
+      COL_ACCOUNT_SETTINGS, &settings,
       -1);
   gtk_list_store_set (GTK_LIST_STORE (model), &iter,
       COL_NAME, new_text,
@@ -1012,7 +1054,7 @@ accounts_dialog_delete_account_response_cb (GtkDialog *message_dialog,
       if (!gtk_tree_selection_get_selected (selection, &model, &iter))
         return;
 
-      gtk_tree_model_get (model, &iter, COL_ACCOUNT_POINTER, &account, -1);
+      gtk_tree_model_get (model, &iter, COL_ACCOUNT, &account, -1);
 
       if (account != NULL)
         {
@@ -1021,7 +1063,17 @@ accounts_dialog_delete_account_response_cb (GtkDialog *message_dialog,
           account = NULL;
         }
 
+      /* No need to call accounts_dialog_model_selection_changed while
+       * removing as we are going to call accounts_dialog_model_select_first
+       * right after which will update the selection. */
+      g_signal_handlers_block_by_func (selection,
+          accounts_dialog_model_selection_changed, account_dialog);
+
       gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+
+      g_signal_handlers_unblock_by_func (selection,
+          accounts_dialog_model_selection_changed, account_dialog);
+
       accounts_dialog_model_select_first (account_dialog);
     }
 
@@ -1039,7 +1091,7 @@ accounts_dialog_remove_account_iter (EmpathyAccountsDialog *dialog,
 
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
 
-  gtk_tree_model_get (model, iter, COL_ACCOUNT_POINTER, &account, -1);
+  gtk_tree_model_get (model, iter, COL_ACCOUNT, &account, -1);
 
   if (account == NULL || !tp_account_is_valid (account))
     {
@@ -1062,6 +1114,7 @@ accounts_dialog_remove_account_iter (EmpathyAccountsDialog *dialog,
       GTK_STOCK_REMOVE, GTK_RESPONSE_YES, NULL);
 
   g_free (question_dialog_primary_text);
+  g_object_unref (account);
 }
 
 static void
@@ -1083,7 +1136,7 @@ accounts_dialog_button_remove_clicked_cb (GtkWidget *button,
   accounts_dialog_remove_account_iter (dialog, &iter);
 }
 
-#ifdef HAVE_MOBLIN
+#ifdef HAVE_MEEGO
 static void
 accounts_dialog_view_delete_activated_cb (EmpathyCellRendererActivatable *cell,
     const gchar *path_string,
@@ -1099,7 +1152,7 @@ accounts_dialog_view_delete_activated_cb (EmpathyCellRendererActivatable *cell,
 
   accounts_dialog_remove_account_iter (dialog, &iter);
 }
-#endif
+#endif /* HAVE_MEEGO */
 
 static void
 accounts_dialog_model_add_columns (EmpathyAccountsDialog *dialog)
@@ -1152,7 +1205,7 @@ accounts_dialog_model_add_columns (EmpathyAccountsDialog *dialog)
       dialog);
   g_object_set (cell, "ypad", 4, NULL);
 
-#ifdef HAVE_MOBLIN
+#ifdef HAVE_MEEGO
   /* Delete column */
   cell = empathy_cell_renderer_activatable_new ();
   gtk_tree_view_column_pack_start (column, cell, FALSE);
@@ -1164,7 +1217,7 @@ accounts_dialog_model_add_columns (EmpathyAccountsDialog *dialog)
   g_signal_connect (cell, "path-activated",
       G_CALLBACK (accounts_dialog_view_delete_activated_cb),
       dialog);
-#endif
+#endif /* HAVE_MEEGO */
 }
 
 static EmpathyAccountSettings *
@@ -1184,7 +1237,7 @@ accounts_dialog_model_get_selected_settings (EmpathyAccountsDialog *dialog)
     return NULL;
 
   gtk_tree_model_get (model, &iter,
-      COL_ACCOUNT_SETTINGS_POINTER, &settings, -1);
+      COL_ACCOUNT_SETTINGS, &settings, -1);
 
   return settings;
 }
@@ -1198,6 +1251,7 @@ accounts_dialog_model_selection_changed (GtkTreeSelection *selection,
   GtkTreeModel *model;
   GtkTreeIter   iter;
   gboolean      is_selection;
+  gboolean creating = FALSE;
 
   is_selection = gtk_tree_selection_get_selected (selection, &model, &iter);
 
@@ -1207,8 +1261,14 @@ accounts_dialog_model_selection_changed (GtkTreeSelection *selection,
   if (settings != NULL)
     g_object_unref (settings);
 
+  if (priv->setting_widget_object != NULL)
+    {
+      g_object_get (priv->setting_widget_object,
+          "creating-account", &creating, NULL);
+    }
+
   /* Update remove button sensitivity */
-  gtk_widget_set_sensitive (priv->button_remove, is_selection);
+  gtk_widget_set_sensitive (priv->button_remove, is_selection && !creating);
 }
 
 static void
@@ -1263,6 +1323,7 @@ accounts_dialog_account_selection_change (GtkTreeSelection *selection,
   TpAccount *account = NULL;
   EmpathyAccountsDialog *dialog = EMPATHY_ACCOUNTS_DIALOG (data);
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
+  gboolean ret;
 
   if (priv->force_change_row)
     {
@@ -1291,13 +1352,17 @@ accounts_dialog_account_selection_change (GtkTreeSelection *selection,
           GTK_STOCK_DISCARD, GTK_RESPONSE_YES, NULL);
 
       g_free (question_dialog_primary_text);
+      ret = FALSE;
     }
   else
     {
-      return TRUE;
+      ret = TRUE;
     }
 
-  return FALSE;
+  if (account != NULL)
+    g_object_unref (account);
+
+  return ret;
 }
 
 static void
@@ -1357,7 +1422,7 @@ accounts_dialog_get_settings_iter (EmpathyAccountsDialog *dialog,
       gboolean   equal;
 
       gtk_tree_model_get (model, iter,
-          COL_ACCOUNT_SETTINGS_POINTER, &this_settings,
+          COL_ACCOUNT_SETTINGS, &this_settings,
           -1);
 
       equal = (this_settings == settings);
@@ -1394,7 +1459,7 @@ accounts_dialog_get_account_iter (EmpathyAccountsDialog *dialog,
       gboolean   equal;
 
       gtk_tree_model_get (model, iter,
-          COL_ACCOUNT_SETTINGS_POINTER, &settings,
+          COL_ACCOUNT_SETTINGS, &settings,
           -1);
 
       equal = empathy_account_settings_has_account (settings, account);
@@ -1436,7 +1501,7 @@ accounts_dialog_add (EmpathyAccountsDialog *dialog,
   gtk_list_store_set (GTK_LIST_STORE (model), &iter,
       COL_NAME, name,
       COL_STATUS, TP_CONNECTION_STATUS_DISCONNECTED,
-      COL_ACCOUNT_SETTINGS_POINTER, settings,
+      COL_ACCOUNT_SETTINGS, settings,
       -1);
 }
 
@@ -1578,8 +1643,8 @@ accounts_dialog_add_account (EmpathyAccountsDialog *dialog,
   gtk_list_store_set (GTK_LIST_STORE (model), &iter,
       COL_NAME, name,
       COL_STATUS, status,
-      COL_ACCOUNT_POINTER, account,
-      COL_ACCOUNT_SETTINGS_POINTER, settings,
+      COL_ACCOUNT, account,
+      COL_ACCOUNT_SETTINGS, settings,
       -1);
 
   accounts_dialog_connection_changed_cb (account,
@@ -1625,6 +1690,37 @@ accounts_dialog_account_validity_changed_cb (TpAccountManager *manager,
   tp_account_prepare_async (account, NULL, account_prepare_cb, dialog);
 }
 
+static void
+accounts_dialog_accounts_model_row_inserted_cb (GtkTreeModel *model,
+    GtkTreePath *path,
+    GtkTreeIter *iter,
+    EmpathyAccountsDialog *dialog)
+{
+  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
+
+  if (priv->setting_widget_object != NULL &&
+      accounts_dialog_has_valid_accounts (dialog))
+    {
+      empathy_account_widget_set_other_accounts_exist (
+          priv->setting_widget_object, TRUE);
+    }
+}
+
+static void
+accounts_dialog_accounts_model_row_deleted_cb (GtkTreeModel *model,
+    GtkTreePath *path,
+    EmpathyAccountsDialog *dialog)
+{
+  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
+
+  if (priv->setting_widget_object != NULL &&
+      !accounts_dialog_has_valid_accounts (dialog))
+    {
+      empathy_account_widget_set_other_accounts_exist (
+          priv->setting_widget_object, FALSE);
+    }
+}
+
 static void
 accounts_dialog_account_removed_cb (TpAccountManager *manager,
     TpAccount *account,
@@ -1636,7 +1732,7 @@ accounts_dialog_account_removed_cb (TpAccountManager *manager,
   if (accounts_dialog_get_account_iter (dialog, account, &iter))
     {
       gtk_list_store_remove (GTK_LIST_STORE (
-            gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview))), &iter);
+          gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview))), &iter);
     }
 }
 
@@ -1651,9 +1747,8 @@ enable_or_disable_account (EmpathyAccountsDialog *dialog,
   /* Update the status in the model */
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
 
-  /* Update the status-infobar in the details view when disabling */
-  if (!enabled)
-    accounts_dialog_update_status_infobar (dialog, account);
+  /* Update the status-infobar in the details view */
+  accounts_dialog_update_status_infobar (dialog, account);
 
   DEBUG ("Account %s is now %s",
       tp_account_get_display_name (account),
@@ -1839,6 +1934,9 @@ dialog_response_cb (GtkWidget *widget,
         {
           gtk_widget_destroy (widget);
         }
+
+      if (account != NULL)
+        g_object_unref (account);
     }
 }
 
@@ -1868,6 +1966,8 @@ accounts_dialog_build_ui (EmpathyAccountsDialog *dialog)
       NULL);
   g_free (filename);
 
+  gtk_widget_set_no_show_all (priv->frame_no_protocol, TRUE);
+
   empathy_builder_connect (gui, dialog,
       "button_add", "clicked", accounts_dialog_button_add_clicked_cb,
       "button_remove", "clicked", accounts_dialog_button_remove_clicked_cb,
@@ -1882,10 +1982,10 @@ accounts_dialog_build_ui (EmpathyAccountsDialog *dialog)
 
   action_area = gtk_dialog_get_action_area (GTK_DIALOG (dialog));
 
-#ifdef HAVE_MOBLIN
+#ifdef HAVE_MEEGO
   gtk_widget_hide (action_area);
   gtk_widget_hide (priv->button_remove);
-#endif
+#endif /* HAVE_MEEGO */
 
   /* Remove button is unsensitive until we have a selected account */
   gtk_widget_set_sensitive (priv->button_remove, FALSE);
@@ -1946,6 +2046,7 @@ accounts_dialog_build_ui (EmpathyAccountsDialog *dialog)
             TP_CONNECTION_PRESENCE_TYPE_OFFLINE), GTK_ICON_SIZE_SMALL_TOOLBAR);
 
   priv->label_status = gtk_label_new (NULL);
+  gtk_label_set_line_wrap (GTK_LABEL (priv->label_status), TRUE);
   gtk_widget_show (priv->label_status);
 
   gtk_box_pack_start (GTK_BOX (hbox), priv->throbber, FALSE, FALSE, 0);
@@ -1982,6 +2083,7 @@ do_dispose (GObject *obj)
 {
   EmpathyAccountsDialog *dialog = EMPATHY_ACCOUNTS_DIALOG (obj);
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
+  GtkTreeModel *model;
 
   if (priv->dispose_has_run)
     return;
@@ -1989,6 +2091,12 @@ do_dispose (GObject *obj)
   priv->dispose_has_run = TRUE;
 
   /* Disconnect signals */
+  model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
+  g_signal_handlers_disconnect_by_func (model,
+      accounts_dialog_accounts_model_row_inserted_cb, dialog);
+  g_signal_handlers_disconnect_by_func (model,
+      accounts_dialog_accounts_model_row_deleted_cb, dialog);
+
   g_signal_handlers_disconnect_by_func (priv->account_manager,
       accounts_dialog_account_validity_changed_cb,
       dialog);
@@ -2027,30 +2135,6 @@ do_dispose (GObject *obj)
   G_OBJECT_CLASS (empathy_accounts_dialog_parent_class)->dispose (obj);
 }
 
-static GObject *
-do_constructor (GType type,
-    guint n_props,
-    GObjectConstructParam *props)
-{
-  GObject *retval;
-
-  if (dialog_singleton)
-    {
-      retval = G_OBJECT (dialog_singleton);
-    }
-  else
-    {
-      retval =
-        G_OBJECT_CLASS (empathy_accounts_dialog_parent_class)->constructor
-            (type, n_props, props);
-
-      dialog_singleton = EMPATHY_ACCOUNTS_DIALOG (retval);
-      g_object_add_weak_pointer (retval, (gpointer) &dialog_singleton);
-    }
-
-  return retval;
-}
-
 static void
 do_get_property (GObject *object,
     guint property_id,
@@ -2093,10 +2177,17 @@ do_constructed (GObject *object)
   EmpathyAccountsDialog *dialog = EMPATHY_ACCOUNTS_DIALOG (object);
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
   gboolean import_asked;
+  GtkTreeModel *model;
 
   accounts_dialog_build_ui (dialog);
   accounts_dialog_model_setup (dialog);
 
+  model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
+  g_signal_connect (model, "row-inserted",
+      (GCallback) accounts_dialog_accounts_model_row_inserted_cb, dialog);
+  g_signal_connect (model, "row-deleted",
+      (GCallback) accounts_dialog_accounts_model_row_deleted_cb, dialog);
+
   /* Set up signalling */
   priv->account_manager = tp_account_manager_dup ();
 
@@ -2129,7 +2220,6 @@ empathy_accounts_dialog_class_init (EmpathyAccountsDialogClass *klass)
   GObjectClass *oclass = G_OBJECT_CLASS (klass);
   GParamSpec *param_spec;
 
-  oclass->constructor = do_constructor;
   oclass->dispose = do_dispose;
   oclass->constructed = do_constructed;
   oclass->set_property = do_set_property;
@@ -2184,3 +2274,73 @@ empathy_accounts_dialog_show (GtkWindow *parent,
 
   return GTK_WIDGET (dialog);
 }
+
+void
+empathy_accounts_dialog_show_application (GdkScreen *screen,
+    GChildWatchFunc application_exit_cb,
+    gpointer user_data,
+    TpAccount *selected_account,
+    gboolean if_needed,
+    gboolean hidden)
+{
+  gint command_pid;
+  GError *error = NULL;
+  gchar *argv[4] = { NULL, };
+  gint i = 0;
+  gchar *account_option = NULL;
+  gchar *path;
+
+  g_return_if_fail (GDK_IS_SCREEN (screen));
+  g_return_if_fail (!selected_account || TP_IS_ACCOUNT (selected_account));
+
+  /* Try to run from source directory if possible */
+  path = g_build_filename (g_getenv ("EMPATHY_SRCDIR"), "src",
+      "empathy-accounts", NULL);
+
+  if (!g_file_test (path, G_FILE_TEST_EXISTS))
+    {
+      g_free (path);
+      path = g_build_filename (BIN_DIR, "empathy-accounts", NULL);
+    }
+
+  argv[i++] = path;
+
+  if (selected_account)
+    {
+      const gchar *account_path;
+
+      account_path = tp_proxy_get_object_path (TP_PROXY (selected_account));
+      account_option = g_strdup_printf ("--select-account=%s",
+          &account_path[strlen (TP_ACCOUNT_OBJECT_PATH_BASE)]);
+
+      argv[i++] = account_option;
+    }
+
+  if (if_needed)
+    argv[i++] = "--if-needed";
+
+  if (hidden)
+    argv[i++] = "--hidden";
+
+  DEBUG ("Launching empathy-accounts (if_needed: %d, hidden: %d, account: %s)",
+    if_needed, hidden,
+    selected_account == NULL ? "<none selected>" :
+      tp_proxy_get_object_path (TP_PROXY (selected_account)));
+
+  gdk_spawn_on_screen (screen, NULL, argv, NULL,
+      G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL,
+      &command_pid, &error);
+  if (error)
+    {
+      g_warning ("Failed to open accounts dialog: %s", error->message);
+      g_error_free (error);
+    }
+
+  /* XXX: unportable cast to GPid; then again, gdk_spawn_on_screen() seems
+   * unportable since it always takes a gint* for the PID */
+  if (application_exit_cb)
+    g_child_watch_add ((GPid) command_pid, application_exit_cb, NULL);
+
+  g_free (account_option);
+  g_free (path);
+}
