@@ -253,9 +253,9 @@ chat_window_create_label (EmpathyChatWindow *window,
 
 	gtk_misc_set_padding (GTK_MISC (name_label), 2, 0);
 	gtk_misc_set_alignment (GTK_MISC (name_label), 0.0, 0.5);
-	g_object_set_data_full (G_OBJECT (chat),
+	g_object_set_data (G_OBJECT (chat),
 		is_tab_label ? "chat-window-tab-label" : "chat-window-menu-label",
-		g_object_ref (name_label), g_object_unref);
+		name_label);
 
 	status_image = gtk_image_new ();
 
@@ -265,12 +265,12 @@ chat_window_create_label (EmpathyChatWindow *window,
 	gtk_box_pack_start (GTK_BOX (event_box_hbox), status_image, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (event_box_hbox), name_label, TRUE, TRUE, 0);
 
-	g_object_set_data_full (G_OBJECT (chat),
+	g_object_set_data (G_OBJECT (chat),
 		is_tab_label ? "chat-window-tab-image" : "chat-window-menu-image",
-		g_object_ref (status_image), g_object_unref);
-	g_object_set_data_full (G_OBJECT (chat),
+		status_image);
+	g_object_set_data (G_OBJECT (chat),
 		is_tab_label ? "chat-window-tab-tooltip-widget" : "chat-window-menu-tooltip-widget",
-		g_object_ref (event_box), g_object_unref);
+		event_box);
 
 	gtk_container_add (GTK_CONTAINER (event_box), event_box_hbox);
 	gtk_box_pack_start (GTK_BOX (hbox), event_box, TRUE, TRUE, 0);
@@ -278,8 +278,7 @@ chat_window_create_label (EmpathyChatWindow *window,
 	if (is_tab_label) {
 		close_button = gtk_button_new ();
 		gtk_button_set_relief (GTK_BUTTON (close_button), GTK_RELIEF_NONE);
-		g_object_set_data_full (G_OBJECT (chat), "chat-window-tab-close-button",
-			g_object_ref (close_button), g_object_unref);
+		g_object_set_data (G_OBJECT (chat), "chat-window-tab-close-button", close_button);
 
 		/* We don't want focus/keynav for the button to avoid clutter, and
 		 * Ctrl-W works anyway.
@@ -1316,6 +1315,8 @@ chat_window_new_message_cb (EmpathyChat       *chat,
 
 	if (has_focus && priv->current_chat == chat) {
 		/* window and tab are focused so consider the message to be read */
+
+		/* FIXME: see Bug#610994 and coments about it in EmpathyChatPriv */
 		empathy_chat_messages_read (chat);
 		return;
 	}
@@ -1530,6 +1531,28 @@ chat_window_focus_in_event_cb (GtkWidget        *widget,
 	return FALSE;
 }
 
+static gboolean
+chat_window_drag_drop (GtkWidget        *widget,
+			 GdkDragContext   *context,
+			 int               x,
+			 int               y,
+			 guint             time_,
+			 gpointer          user_data)
+{
+	GdkAtom target, uri_target, contact_target;
+
+	target = gtk_drag_dest_find_target (widget, context, NULL);
+	uri_target = gdk_atom_intern_static_string ("text/uri-list");
+	contact_target = gdk_atom_intern_static_string ("text/contact-id");
+
+	if (target == uri_target || target == contact_target) {
+		gtk_drag_get_data (widget, context, target, time_);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static gboolean
 chat_window_drag_motion (GtkWidget        *widget,
 			 GdkDragContext   *context,
@@ -1587,9 +1610,7 @@ chat_window_drag_motion (GtkWidget        *widget,
 		return TRUE;
 	}
 
-	/* Otherwise, it must be a notebook tab drag.  Set to MOVE. */
-	gdk_drag_status (context, GDK_ACTION_MOVE, time_);
-	return TRUE;
+	return FALSE;
 }
 
 static void
@@ -1708,23 +1729,10 @@ chat_window_drag_data_received (GtkWidget        *widget,
 			EmpathyChatWindowPriv *priv;
 
 			priv = GET_PRIV (window);
-
-			if (old_window == window) {
-				DEBUG ("DND tab (within same window)");
-				priv->dnd_same_window = TRUE;
-				gtk_drag_finish (context, TRUE, FALSE, time_);
-				return;
-			}
-
-			priv->dnd_same_window = FALSE;
+			priv->dnd_same_window = (old_window == window);
+			DEBUG ("DND tab (within same window: %s)",
+				priv->dnd_same_window ? "Yes" : "No");
 		}
-
-		/* We should return TRUE to remove the data when doing
-		 * GDK_ACTION_MOVE, but we don't here otherwise it has
-		 * weird consequences, and we handle that internally
-		 * anyway with add_chat () and remove_chat ().
-		 */
-		gtk_drag_finish (context, TRUE, FALSE, time_);
 	} else {
 		DEBUG ("DND from unknown source");
 		gtk_drag_finish (context, FALSE, FALSE, time_);
@@ -1921,7 +1929,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 
 	/* Set up drag and drop */
 	gtk_drag_dest_set (GTK_WIDGET (priv->notebook),
-			   GTK_DEST_DEFAULT_ALL,
+			   GTK_DEST_DEFAULT_HIGHLIGHT,
 			   drag_types_dest,
 			   G_N_ELEMENTS (drag_types_dest),
 			   GDK_ACTION_MOVE | GDK_ACTION_COPY);
@@ -1935,6 +1943,10 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 			  "drag-data-received",
 			  G_CALLBACK (chat_window_drag_data_received),
 			  window);
+	g_signal_connect (priv->notebook,
+			  "drag-drop",
+			  G_CALLBACK (chat_window_drag_drop),
+			  window);
 
 	chat_windows = g_list_prepend (chat_windows, window);
 
@@ -2021,6 +2033,7 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 	GtkWidget             *label;
 	GtkWidget             *popup_label;
 	GtkWidget             *child;
+	GValue                value = { 0, };
 
 	g_return_if_fail (window != NULL);
 	g_return_if_fail (EMPATHY_IS_CHAT (chat));
@@ -2068,8 +2081,13 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 	gtk_notebook_append_page_menu (GTK_NOTEBOOK (priv->notebook), child, label, popup_label);
 	gtk_notebook_set_tab_reorderable (GTK_NOTEBOOK (priv->notebook), child, TRUE);
 	gtk_notebook_set_tab_detachable (GTK_NOTEBOOK (priv->notebook), child, TRUE);
-	gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (priv->notebook), child,
-					    TRUE, TRUE, GTK_PACK_START);
+	g_value_init (&value, G_TYPE_BOOLEAN);
+	g_value_set_boolean (&value, TRUE);
+	gtk_container_child_set_property (GTK_CONTAINER (priv->notebook),
+					  child, "tab-expand" , &value);
+	gtk_container_child_set_property (GTK_CONTAINER (priv->notebook),
+					  child,  "tab-fill" , &value);
+	g_value_unset (&value);
 
 	DEBUG ("Chat added (%d references)", G_OBJECT (chat)->ref_count);
 }
