@@ -44,6 +44,13 @@
 #include <telepathy-glib/connection-manager.h>
 #include <telepathy-glib/interfaces.h>
 
+#ifdef ENABLE_TPL
+#include <telepathy-logger/log-manager.h>
+#include <telepathy-logger/log-store-empathy.h>
+#else
+
+#include <libempathy/empathy-log-manager.h>
+#endif /* ENABLE_TPL */
 #include <libempathy/empathy-idle.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-call-factory.h>
@@ -53,7 +60,6 @@
 #include <libempathy/empathy-connection-managers.h>
 #include <libempathy/empathy-dispatcher.h>
 #include <libempathy/empathy-dispatch-operation.h>
-#include <libempathy/empathy-log-manager.h>
 #include <libempathy/empathy-ft-factory.h>
 #include <libempathy/empathy-tp-chat.h>
 #include <libempathy/empathy-tp-call.h>
@@ -63,6 +69,8 @@
 #include <libempathy-gtk/empathy-location-manager.h>
 
 #include "empathy-main-window.h"
+#include "empathy-import-mc4-accounts.h"
+#include "empathy-accounts-common.h"
 #include "empathy-accounts-dialog.h"
 #include "empathy-status-icon.h"
 #include "empathy-call-window.h"
@@ -79,6 +87,10 @@
 static gboolean start_hidden = FALSE;
 static gboolean no_connect = FALSE;
 
+static void account_manager_ready_cb (GObject *source_object,
+    GAsyncResult *result,
+    gpointer user_data);
+
 static void
 dispatch_cb (EmpathyDispatcher *dispatcher,
     EmpathyDispatchOperation *operation,
@@ -242,10 +254,10 @@ accounts_application_exited_cb (GPid pid,
 
 static void
 show_accounts_ui (GdkScreen *screen,
-    gboolean try_import)
+    gboolean if_needed)
 {
   empathy_accounts_dialog_show_application (screen,
-      accounts_application_exited_cb, NULL, NULL, try_import, start_hidden);
+      accounts_application_exited_cb, NULL, NULL, if_needed, start_hidden);
 }
 
 static UniqueResponse
@@ -256,6 +268,7 @@ unique_app_message_cb (UniqueApp *unique_app,
     gpointer user_data)
 {
   GtkWindow *window = user_data;
+  TpAccountManager *account_manager;
 
   DEBUG ("Other instance launched, presenting the main window. "
       "Command=%d, timestamp %u", command, timestamp);
@@ -273,6 +286,12 @@ unique_app_message_cb (UniqueApp *unique_app,
   gtk_window_set_startup_id (GTK_WINDOW (window),
       unique_message_data_get_startup_id (data));
   gtk_window_present_with_time (GTK_WINDOW (window), timestamp);
+  gtk_window_set_skip_taskbar_hint (window, FALSE);
+
+  account_manager = tp_account_manager_dup ();
+  tp_account_manager_prepare_async (account_manager, NULL,
+      account_manager_ready_cb, NULL);
+  g_object_unref (account_manager);
 
   return UNIQUE_RESPONSE_OK;
 }
@@ -363,7 +382,11 @@ account_manager_ready_cb (GObject *source_object,
       /* if current state is Offline, then put it online */
       empathy_idle_set_state (idle, TP_CONNECTION_PRESENCE_TYPE_AVAILABLE);
 
-  show_accounts_ui (gdk_screen_get_default (), TRUE);
+  /* Pop up the accounts dialog if it's needed (either when we don't have any
+   * non-salut accounts yet or when we haven't imported mc4 accounts yet */
+  if (!empathy_accounts_has_non_salut_accounts (manager)
+      || !empathy_import_mc4_has_imported ())
+    show_accounts_ui (gdk_screen_get_default (), TRUE);
 
   g_object_unref (idle);
   g_object_unref (connectivity);
@@ -547,7 +570,11 @@ main (int argc, char *argv[])
   EmpathyStatusIcon *icon;
   EmpathyDispatcher *dispatcher;
   TpAccountManager *account_manager;
+#ifdef ENABLE_TPL
+  TplLogManager *log_manager;
+#else
   EmpathyLogManager *log_manager;
+#endif /* ENABLE_TPL */
   EmpathyChatroomManager *chatroom_manager;
   EmpathyCallFactory *call_factory;
   EmpathyFTFactory  *ft_factory;
@@ -560,7 +587,7 @@ main (int argc, char *argv[])
 
 #ifdef ENABLE_DEBUG
   TpDebugSender *debug_sender;
-#endif
+#endif /* ENABLE_TPL */
 
   GOptionContext *optcontext;
   GOptionEntry options[] = {
@@ -612,7 +639,7 @@ main (int argc, char *argv[])
   g_log_set_default_handler (tp_debug_sender_log_handler, G_LOG_DOMAIN);
 #endif
 
-  unique_app = unique_app_new (EMPATHY_CLIENT_NAME, NULL);
+  unique_app = unique_app_new ("org.gnome."PACKAGE_NAME, NULL);
 
   if (unique_app_is_running (unique_app))
     {
@@ -654,8 +681,12 @@ main (int argc, char *argv[])
       G_CALLBACK (unique_app_message_cb), window);
 
   /* Logging */
+#ifdef ENABLE_TPL
+  log_manager = tpl_log_manager_dup_singleton ();
+#else
   log_manager = empathy_log_manager_dup_singleton ();
   empathy_log_manager_observe (log_manager, dispatcher);
+#endif
 
   chatroom_manager = empathy_chatroom_manager_dup_singleton (NULL);
   empathy_chatroom_manager_observe (chatroom_manager, dispatcher);
